@@ -11,7 +11,7 @@
  *                                                                           *
  *****************************************************************************
  *                                                                           *
- *   Copyright (c) 2000-2006 Wolfgang Hoermann and Josef Leydold             *
+ *   Copyright (c) 2000-2011 Wolfgang Hoermann and Josef Leydold             *
  *   Department of Statistics and Mathematics, WU Wien, Austria              *
  *                                                                           *
  *   This program is free software; you can redistribute it and/or modify    *
@@ -33,19 +33,22 @@
  *                                                                           *
  * DSTD is a wrapper for special generator for Discrete univariate           *
  * STandarD distributions. It only works for distributions in the UNURAN     *
- * library of distributions and will refuse to work otherwise.               *
- * (In detail it rejects a distribution if its id is equal to DISTR_GENERIC, *
- * the id inserted my the unur_distr_discr_new() call.)                      *
+ * library of distributions or those discrete distributions that have        *
+ * the inverse CDF implemented. Otherwise it refuses to work.                *
  *                                                                           *
- * It calls the initialzation routine provided by the distribution object.   *
+ * It calls the initialization routine provided by the distribution object.  *
  * This routine has to do all setup steps for the special generator.         *
- * if no such routine is given, i.e. distr->init==NULL, then unur_dstd_new() *
- * does not work and the NULL pointer is returned instead of the pointer to  *
- * a parameter object.                                                       *
+ * If no such routine is given (i.e. distr->init==NULL) or when it does not  *
+ * implement the inversion method then the inverse CDF stored in the         *
+ * distribution object (if available) is used.                               *
+ * If neither is available, then unur_dstd_new() does not work and the       *
+ * NULL pointer is returned instead of the pointer to a parameter object.    *
  *                                                                           *
  * Notice that using a truncated distribution (this can be constructed by    *
  * changing the default domain of a distribution by means of an              *
- * unur_distr_cont_set_domain() call) is not allowed.                        *
+ * unur_distr_discr_set_domain() call) is only allowed if the inversion      *
+ * method is used. Otherwise no parameter object is returned by the          *
+ * unur_dstd_new() call.                                                     *
  *                                                                           *
  * Variants (different algorithms for the same distribution) are possible    *
  * and can be selected by unsigned integers using the                        *
@@ -53,11 +56,19 @@
  * For possible variants see the generator files for each distribution in    *
  * the distributions directory. However the following are common to all      *
  * distributions:                                                            *
- *    0                     ... the default generator                        *
+ *                                                                           *
+ *    UNUR_STDGEN_DEFAULT   ... the default generator                        *
  *    UNUR_STDGEN_INVERSION ... the inversion method (if available)          *
+ *    UNUR_STDGEN_FAST      ... the fasted available special generator       *
+ *                                                                           *
  * unur_dstd_set_variant() return 0 if a variant is not implemented, and 1   *
  * otherwise. In the first case the selected variant is not changed.         *
  *                                                                           *
+ * The domain of a (truncated) distribution can be changed without building  *
+ * a new generator object by means of the unur_dstd_chg_truncated() call.    *
+ * Notice that this only works when the inversion method is used. Otherwise  *
+ * nothing happens to the domain and an error message is produced.           *
+ *                                                                           *
  *****************************************************************************/
 
 /*---------------------------------------------------------------------------*/
@@ -67,9 +78,11 @@
 #include <distr/distr_source.h>
 #include <distr/discr.h>
 #include <distributions/unur_stddistr.h>
+#include <distributions/unur_distributions_source.h>
 #include <urng/urng.h>
 #include "unur_methods_source.h"
 #include "x_gen_source.h"
+#include "cstd.h"    /* required for UNUR_STDGEN_* macros */
 #include "dstd.h"
 #include "dstd_struct.h"
 
@@ -87,8 +100,9 @@
 /*    bits 13-24 ... adaptive steps                                          */
 /*    bits 25-32 ... trace sampling                                          */
 
-#define DSTD_DEBUG_REINIT    0x00000010u   /* print parameters after reinit  */
-#define DSTD_DEBUG_CHG       0x00001000u   /* print changed parameters       */
+#define DSTD_DEBUG_GEN       0x00000005u   /* print constants for generator  */
+#define DSTD_DEBUG_REINIT    0x00000010u   /* print params of distr after reinit */
+#define DSTD_DEBUG_CHG       0x00001000u   /* print changed params of distr  */
 
 /*---------------------------------------------------------------------------*/
 /* Flags for logging set calls                                               */
@@ -137,6 +151,16 @@ static void _unur_dstd_free( struct unur_gen *gen);
 /* double _unur_dstd_sample( UNUR_GEN *gen ); does not exist!                */
 /*---------------------------------------------------------------------------*/
 
+static int _unur_dstd_sample_inv( struct unur_gen *gen ); 
+/*---------------------------------------------------------------------------*/
+/* Generic inversion method.                                                 */
+/*---------------------------------------------------------------------------*/
+
+static int _unur_dstd_inversion_init( struct unur_par *par, struct unur_gen *gen );
+/*---------------------------------------------------------------------------*/
+/* Initialize special generator for inversion method.                        */
+/*---------------------------------------------------------------------------*/
+
 #ifdef UNUR_ENABLE_LOGGING
 /*---------------------------------------------------------------------------*/
 /* the following functions print debugging information on output stream,     */
@@ -152,6 +176,11 @@ static void _unur_dstd_debug_chg_pmfparams( const struct unur_gen *gen );
 /*---------------------------------------------------------------------------*/
 /* print new (changed) parameters of distribution                            */
 /*---------------------------------------------------------------------------*/
+
+static void _unur_dstd_debug_chg_truncated( struct unur_gen *gen );
+/*---------------------------------------------------------------------------*/
+/* print new (changed) domain of (truncated) distribution                    */
+/*---------------------------------------------------------------------------*/
 #endif
 
 #ifdef UNUR_ENABLE_INFO
@@ -170,11 +199,10 @@ static void _unur_dstd_info( struct unur_gen *gen, int help );
 #define GEN       ((struct unur_dstd_gen*)gen->datap) /* data for generator object */
 #define DISTR     gen->distr->data.discr /* data for distribution in generator object */
 
-#define BD_LEFT   domain[0]             /* left boundary of domain of distribution */
-#define BD_RIGHT  domain[1]             /* right boundary of domain of distribution */
-
 #define SAMPLE    gen->sample.discr     /* pointer to sampling routine       */
 
+#define CDF(x)    _unur_discr_CDF((x),(gen->distr))   /* call to CDF         */
+
 /*---------------------------------------------------------------------------*/
 
 /*****************************************************************************/
@@ -199,19 +227,15 @@ unur_dstd_new( const struct unur_distr *distr )
   struct unur_par *par;
 
   /* check arguments */
-  _unur_check_NULL( GENTYPE,distr,NULL );
+  _unur_check_NULL(GENTYPE,distr,NULL);
 
   /* check distribution */
   if (distr->type != UNUR_DISTR_DISCR) {
     _unur_error(GENTYPE,UNUR_ERR_DISTR_INVALID,""); return NULL; }
   COOKIE_CHECK(distr,CK_DISTR_DISCR,NULL);
 
-  if (!(distr->id & UNUR_DISTR_STD) ) {
-    _unur_error(GENTYPE,UNUR_ERR_DISTR_INVALID,"standard distribution");
-    return NULL;
-  }
-  if (DISTR_IN.init == NULL) {
-    _unur_error(GENTYPE,UNUR_ERR_DISTR_REQUIRED,"init() for special generators");
+  if (DISTR_IN.init == NULL && DISTR_IN.invcdf == NULL) {
+    _unur_error(GENTYPE,UNUR_ERR_DISTR_REQUIRED,"init() for special generators or inverse CDF");
     return NULL;
   }
 
@@ -225,7 +249,7 @@ unur_dstd_new( const struct unur_distr *distr )
   /* set default values */
   par->method   = UNUR_METH_DSTD;   /* method                                */
   par->variant  = 0u;               /* default variant                       */
-  par->set      = 0u;               /* inidicate default parameters          */    
+  par->set      = 0u;               /* indicate default parameters           */    
   par->urng     = unur_get_default_urng(); /* use default urng               */
   par->urng_aux = NULL;                    /* no auxilliary URNG required    */
 
@@ -266,18 +290,124 @@ unur_dstd_set_variant( struct unur_par *par, unsigned variant )
   par->variant = variant;
 
   /* check variant. run special init routine only in test mode */
-  if (par->DISTR_IN.init != NULL && par->DISTR_IN.init(par,NULL)==UNUR_SUCCESS ) {
-    par->set |= DSTD_SET_VARIANT;    /* changelog */
-    return UNUR_SUCCESS;
+  if ( (par->DISTR_IN.init == NULL || par->DISTR_IN.init(par,NULL)!=UNUR_SUCCESS) &&
+       _unur_dstd_inversion_init(par,NULL)!=UNUR_SUCCESS ) {
+    /* variant not valid */
+    _unur_warning(GENTYPE,UNUR_ERR_PAR_VARIANT,"");
+    par->variant = old_variant;
+    return UNUR_ERR_PAR_VARIANT;
   }
 
-  /* variant not valid */
-  _unur_warning(GENTYPE,UNUR_ERR_PAR_VARIANT,"");
-  par->variant = old_variant;
-  return UNUR_ERR_PAR_VARIANT;
+  /* changelog */
+  par->set |= DSTD_SET_VARIANT;
+
+  /* o.k. */
+  return UNUR_SUCCESS;
 
 } /* end of unur_dstd_set_variant() */
 
+/*---------------------------------------------------------------------------*/
+
+int
+unur_dstd_chg_truncated( struct unur_gen *gen, int left, int right )
+     /*----------------------------------------------------------------------*/
+     /* change the left and right borders of the domain of the               */
+     /* (truncated) distribution                                             */
+     /*                                                                      */
+     /* parameters:                                                          */
+     /*   gen   ... pointer to generator object                              */
+     /*   left  ... left boundary point                                      */
+     /*   right ... right boundary point                                     */
+     /*                                                                      */
+     /* return:                                                              */
+     /*   UNUR_SUCCESS ... on success                                        */
+     /*   error code   ... on error                                          */
+     /*                                                                      */
+     /* comment:                                                             */
+     /*   the new boundary points may be +/- UNUR_INFINITY                   */
+     /*----------------------------------------------------------------------*/
+{
+  double Umin, Umax;
+
+  /* check arguments */
+  _unur_check_NULL( GENTYPE, gen, UNUR_ERR_NULL );
+  _unur_check_gen_object( gen, DSTD, UNUR_ERR_GEN_INVALID );
+
+  /* domain can only be changed for inversion method! */
+  if ( ! GEN->is_inversion ) { 
+    /* this is not the inversion method */
+    _unur_warning(gen->genid,UNUR_ERR_GEN_DATA,"truncated domain for non-inversion method");
+    return UNUR_ERR_GEN_DATA;
+  }
+
+  /* CDF required ! */
+  if (DISTR.cdf == NULL) {
+    _unur_warning(gen->genid,UNUR_ERR_GEN_DATA,"truncated domain, CDF required");
+    return UNUR_ERR_GEN_DATA;
+  }
+
+  /* check new parameter for generator */
+  /* (the truncated domain must be a subset of the domain) */
+  if (left < DISTR.domain[0]) {
+    _unur_warning(NULL,UNUR_ERR_DISTR_SET,"truncated domain too large");
+    left = DISTR.domain[0];
+  }
+  if (right > DISTR.domain[1]) {
+    _unur_warning(NULL,UNUR_ERR_DISTR_SET,"truncated domain too large");
+    right = DISTR.domain[1];
+  }
+
+  if (left >= right) {
+    _unur_warning(NULL,UNUR_ERR_DISTR_SET,"domain, left >= right");
+    return UNUR_ERR_DISTR_SET;
+  }
+
+  /* compute umin and umax */
+  Umin = (left <= INT_MIN) ? 0. : CDF(left-1);
+  Umax = CDF(right);
+
+  /* check result */
+  if (Umin > Umax) {
+    /* this is a serios error that should not happen */
+    _unur_error(gen->genid,UNUR_ERR_SHOULD_NOT_HAPPEN,"");
+    return UNUR_ERR_SHOULD_NOT_HAPPEN;
+  }
+
+  if (_unur_FP_equal(Umin,Umax)) {
+    /* CDF values very close */
+    _unur_warning(gen->genid,UNUR_ERR_DISTR_SET,"CDF values very close");
+    if (_unur_iszero(Umin) || _unur_FP_same(Umax,1.)) {
+      /* this is very bad */
+      _unur_warning(gen->genid,UNUR_ERR_DISTR_SET,"CDF values at boundary points too close");
+      return UNUR_ERR_DISTR_SET;
+    }
+  }
+
+
+  /* copy new boundaries into generator object */
+  DISTR.trunc[0] = left;
+  DISTR.trunc[1] = right;
+  GEN->Umin = Umin;
+  GEN->Umax = Umax;
+
+  /* changelog */
+  gen->distr->set |= UNUR_DISTR_SET_TRUNCATED;
+
+  /* indicate that we have a truncated distribution.
+     (do not have the standard domain any more) */
+  gen->distr->set &= ~UNUR_DISTR_SET_STDDOMAIN;
+
+#ifdef UNUR_ENABLE_LOGGING
+  /* write info into LOG file */
+  if (gen->debug & DSTD_DEBUG_CHG)
+    _unur_dstd_debug_chg_truncated( gen );
+#endif
+  
+  /* o.k. */
+  return UNUR_SUCCESS;
+  
+} /* end of unur_dstd_chg_truncated() */
+
 
 /*****************************************************************************/
 /**  Private                                                                **/
@@ -302,11 +432,6 @@ _unur_dstd_init( struct unur_par *par )
 
   /* check arguments */
   CHECK_NULL(par,NULL);
-  /* check for required data: initializing routine for special generator */
-  if (par->DISTR_IN.init == NULL) {
-    _unur_error(GENTYPE,UNUR_ERR_NULL,"");
-    return NULL;
-  }
 
   /* check input */
   if ( par->method != UNUR_METH_DSTD ) {
@@ -320,19 +445,20 @@ _unur_dstd_init( struct unur_par *par )
   _unur_par_free(par);
   if (!gen) return NULL;
 
-  /* check parameters */
-  if (_unur_dstd_check_par(gen) != UNUR_SUCCESS) {
-    _unur_dstd_free(gen); return NULL;
-  }
-
   /* run special init routine for generator */
   GEN->is_inversion = FALSE;   /* reset flag for inversion method */
-  if ( DISTR.init(NULL,gen)!=UNUR_SUCCESS ) {
+  if ( (DISTR.init == NULL || DISTR.init(NULL,gen)!=UNUR_SUCCESS) &&
+       _unur_dstd_inversion_init(NULL,gen)!=UNUR_SUCCESS ) {
     /* init failed --> could not find a sampling routine */
     _unur_error(GENTYPE,UNUR_ERR_GEN_DATA,"variant for special generator");
     _unur_dstd_free(gen); return NULL; 
   }
 
+  /* check parameters */
+  if (_unur_dstd_check_par(gen) != UNUR_SUCCESS) {
+    _unur_dstd_free(gen); return NULL;
+  }
+
 #ifdef UNUR_ENABLE_LOGGING
   /* write info into LOG file */
   if (gen->debug) _unur_dstd_debug_init(gen);
@@ -360,18 +486,19 @@ _unur_dstd_reinit( struct unur_gen *gen )
 {
   int rcode;
 
-  /* check parameters */
-  if ( (rcode = _unur_dstd_check_par(gen)) != UNUR_SUCCESS)
-    return rcode;
-
   /* run special init routine for generator */
   GEN->is_inversion = FALSE;   /* reset flag for inversion method */
-  if ( DISTR.init(NULL,gen)!=UNUR_SUCCESS ) {
+  if ( (DISTR.init == NULL || DISTR.init(NULL,gen)!=UNUR_SUCCESS) &&
+       _unur_dstd_inversion_init(NULL,gen)!=UNUR_SUCCESS ) {
     /* init failed --> could not find a sampling routine */
     _unur_error(gen->genid,UNUR_ERR_GEN_DATA,"parameters");
     return UNUR_ERR_GEN_DATA;
   }
 
+  /* check parameters */
+  if ( (rcode = _unur_dstd_check_par(gen)) != UNUR_SUCCESS)
+    return rcode;
+
 #ifdef UNUR_ENABLE_LOGGING
     /* write info into LOG file */
     if (gen->debug & DSTD_DEBUG_REINIT)
@@ -428,8 +555,10 @@ _unur_dstd_create( struct unur_par *par )
   GEN->sample_routine_name = NULL ;  /* name of sampling routine  */
 
   /* copy some parameters into generator object */
-  GEN->Umin        = 0;    /* cdf at left boundary of domain   */
-  GEN->Umax        = 1;    /* cdf at right boundary of domain  */
+  GEN->Umin = 0.;               /* cdf at left boundary of domain            */
+  GEN->Umax = 1.;               /* cdf at right boundary of domain           */
+
+  /* GEN->is_inversion is set in _unur_dstd_inversion_init() */
 
 #ifdef UNUR_ENABLE_INFO
   /* set function for creating info string */
@@ -456,11 +585,28 @@ _unur_dstd_check_par( struct unur_gen *gen )
      /*   error code   ... on error                                          */
      /*----------------------------------------------------------------------*/
 {
-  /* domain must not be changed */
+  /* domain valid for special generator ?? */
   if (!(gen->distr->set & UNUR_DISTR_SET_STDDOMAIN)) {
-    /* domain has been modified --> not allowed */
-    _unur_error(gen->genid,UNUR_ERR_GEN_DATA,"domain changed");
-    return UNUR_ERR_GEN_DATA;
+    /* domain has been modified */
+    gen->distr->set &= UNUR_DISTR_SET_TRUNCATED;
+    DISTR.trunc[0] = DISTR.domain[0];
+    DISTR.trunc[1] = DISTR.domain[1];
+
+    if ( ! GEN->is_inversion ) {
+      /* this is not the inversion method */
+      _unur_error(gen->genid,UNUR_ERR_GEN_DATA,"domain changed for non inversion method");
+      return UNUR_ERR_GEN_DATA;
+    }
+
+    if (DISTR.cdf == NULL) {
+      /* using a truncated distribution requires a CDF */
+      _unur_error(gen->genid,UNUR_ERR_GEN_DATA,"domain changed, CDF required");
+      return UNUR_ERR_GEN_DATA;
+    }
+
+    /* compute Umin and Umax */
+    GEN->Umin = (DISTR.trunc[0] <= INT_MIN) ? 0. : CDF(DISTR.trunc[0]-1);
+    GEN->Umax = CDF(DISTR.trunc[1]);
   }
 
   return UNUR_SUCCESS;
@@ -551,10 +697,142 @@ _unur_dstd_free( struct unur_gen *gen )
     Sampling routines are defined in ../distributions/ for each distributions.
 **/
 
+/*****************************************************************************/
+
+int
+_unur_dstd_sample_inv( struct unur_gen *gen ) 
+     /*----------------------------------------------------------------------*/
+     /* generic inversion method.                                            */
+     /*                                                                      */
+     /* parameters:                                                          */
+     /*   gen ... pointer to generator object                                */
+     /*                                                                      */
+     /* return:                                                              */
+     /*   double (sample from random variate)                                */
+     /*                                                                      */
+     /* error:                                                               */
+     /*   return INT_MAX                                                     */
+     /*----------------------------------------------------------------------*/
+{
+  double U;
+
+  if (!DISTR.invcdf) return INT_MAX;
+
+  /* sample from uniform random number generator */
+  while (_unur_iszero(U = GEN->Umin + _unur_call_urng(gen->urng) * (GEN->Umax-GEN->Umin)));
+
+  /* compute inverse CDF */
+  return ((int) DISTR.invcdf(U,gen->distr));
+
+} /* _unur_dstd_sample_inv() */
+
+/*---------------------------------------------------------------------------*/
+
+int
+unur_dstd_eval_invcdf( const struct unur_gen *gen, double u )
+     /*----------------------------------------------------------------------*/
+     /* evaluate inverse CDF at u.                                           */
+     /*                                                                      */
+     /* parameters:                                                          */
+     /*   gen ... pointer to generator object                                */
+     /*   u   ... argument for inverse CDF (0<=u<=1, no validation!)         */
+     /*                                                                      */
+     /* return:                                                              */
+     /*   int (inverse CDF)                                                  */
+     /*                                                                      */
+     /* error:                                                               */
+     /*   return INT_MAX                                                     */
+     /*----------------------------------------------------------------------*/
+{
+  int k;
+
+  /* check arguments */
+  _unur_check_NULL( GENTYPE, gen, INT_MAX );
+  if ( gen->method != UNUR_METH_DSTD ) {
+    _unur_error(gen->genid,UNUR_ERR_GEN_INVALID,"");
+    return INT_MAX;
+  }
+  COOKIE_CHECK(gen,CK_DSTD_GEN,INT_MAX);
+
+  if (!DISTR.invcdf) {
+    /* no inverse CDF available */
+    _unur_error(gen->genid,UNUR_ERR_NO_QUANTILE,"inversion CDF required");
+    return INT_MAX;
+  } 
+
+  if ( ! (u>0. && u<1.)) {
+    if ( ! (u>=0. && u<=1.)) {
+      _unur_warning(gen->genid,UNUR_ERR_DOMAIN,"U not in [0,1]");
+    }
+    if (u<=0.) return DISTR.trunc[0];
+    if (u>=1.) return DISTR.trunc[1];
+    return INT_MAX;  /* u == NaN */
+  }
+  
+  /* rescale given u */
+  u = GEN->Umin + u * (GEN->Umax - GEN->Umin);
+
+  /* compute inverse CDF */
+  k = DISTR.invcdf(u,gen->distr);
+
+  /* validate range */
+  if (k<DISTR.trunc[0]) k = DISTR.trunc[0];
+  if (k>DISTR.trunc[1]) k = DISTR.trunc[1];
+
+  return k;
+
+} /* end of unur_dstd_eval_invcdf() */
+
+
 /*****************************************************************************/
 /**  Auxilliary Routines                                                    **/
 /*****************************************************************************/
 
+int
+_unur_dstd_inversion_init( struct unur_par *par, struct unur_gen *gen )
+     /*----------------------------------------------------------------------*/
+     /* initialize special generator for inversion method                    */
+     /* when inverse CDF is available for the distribution.                  */
+     /* if gen == NULL then only check existance of variant.                 */
+     /*                                                                      */
+     /* parameters:                                                          */
+     /*   par ... pointer to parameter for building generator object         */
+     /*   gen ... pointer to generator object                                */
+     /*                                                                      */
+     /* return:                                                              */
+     /*   UNUR_SUCCESS ... on success                                        */
+     /*   error code   ... on error                                          */
+     /*----------------------------------------------------------------------*/
+{
+  /* one of par and gen must not be the NULL pointer */
+  switch ((par) ? par->variant : gen->variant) {
+
+  case 0:  /* DEFAULT */
+  case UNUR_STDGEN_INVERSION:   /* inversion method */
+    if (gen) {
+      if (DISTR.invcdf) {
+	GEN->is_inversion = TRUE;
+	_unur_dstd_set_sampling_routine(gen,_unur_dstd_sample_inv);
+	return UNUR_SUCCESS;
+      }
+    }
+    else {
+      if ((par->distr->data.discr).invcdf) {
+	return UNUR_SUCCESS;
+      }
+    }
+
+    /* FALLTHROUGH */
+  default: /* no such generator */
+    if (gen) _unur_warning(gen->genid,UNUR_ERR_SHOULD_NOT_HAPPEN,"");
+    return UNUR_FAILURE;
+  }
+  
+} /* end of _unur_dstd_inversion_init() */
+
+/*---------------------------------------------------------------------------*/
+
+
 /*****************************************************************************/
 /**  Debugging utilities                                                    **/
 /*****************************************************************************/
@@ -573,6 +851,7 @@ _unur_dstd_debug_init( const struct unur_gen *gen )
      /*----------------------------------------------------------------------*/
 {
   FILE *LOG;
+  int i;
 
   /* check arguments */
   CHECK_NULL(gen,RETURN_VOID);  COOKIE_CHECK(gen,CK_DSTD_GEN,RETURN_VOID);
@@ -597,12 +876,39 @@ _unur_dstd_debug_init( const struct unur_gen *gen )
     fprintf(LOG,"   (Inversion)");
   fprintf(LOG,"\n%s:\n",gen->genid);
 
+  /* table of precomputed constants for special generators */
+  if (gen->debug & DSTD_DEBUG_GEN) {
+    fprintf(LOG,"%s: precomputed double constants for routine: ",gen->genid);
+    if (GEN->gen_param) {
+      fprintf(LOG,"%d\n",GEN->n_gen_param);
+      for (i=0; i < GEN->n_gen_param; i++)
+     	fprintf(LOG,"%s:\t[%d] = %g\n",gen->genid,i,GEN->gen_param[i]);
+    }
+    else {
+      fprintf(LOG,"none\n");
+    }
+
+    fprintf(LOG,"%s: precomputed integer constants for routine: ",gen->genid);
+    if (GEN->gen_iparam) {
+      fprintf(LOG,"%d\n",GEN->n_gen_iparam);
+      for (i=0; i < GEN->n_gen_iparam; i++)
+   	fprintf(LOG,"%s:\t[%d] = %d\n",gen->genid,i,GEN->gen_iparam[i]);
+    }
+    else {
+      fprintf(LOG,"none\n");
+    }
+    fprintf(LOG,"%s:\n",gen->genid);
+  }
+
+  /* truncated domain ? */
   if (!(gen->distr->set & UNUR_DISTR_SET_STDDOMAIN)) {
     fprintf(LOG,"%s: domain has been changed. U in (%g,%g)\n",gen->genid,GEN->Umin,GEN->Umax);
     fprintf(LOG,"%s:\n",gen->genid);
   }
 
-} /* end of _unur_dstd_info_init() */
+  fflush(LOG);
+
+} /* end of _unur_dstd_debug_init() */
 
 /*---------------------------------------------------------------------------*/
 
@@ -629,6 +935,30 @@ _unur_dstd_debug_chg_pmfparams( const struct unur_gen *gen )
 
 } /* end of _unur_dstd_debug_chg_pmfparams() */
 
+/*---------------------------------------------------------------------------*/
+
+void 
+_unur_dstd_debug_chg_truncated( struct unur_gen *gen )
+     /*----------------------------------------------------------------------*/
+     /* print new (changed) domain of (truncated) distribution               */
+     /*                                                                      */
+     /* parameters:                                                          */
+     /*   gen ... pointer to generator object                                */
+     /*----------------------------------------------------------------------*/
+{
+  FILE *LOG;
+
+  /* check arguments */
+  CHECK_NULL(gen,RETURN_VOID);  COOKIE_CHECK(gen,CK_DSTD_GEN,RETURN_VOID);
+
+  LOG = unur_get_stream();
+
+  fprintf(LOG,"%s: domain of truncated distribution changed:\n",gen->genid);
+  fprintf(LOG,"%s:\tdomain = (%d, %d)\n",gen->genid, DISTR.trunc[0], DISTR.trunc[1]);
+  fprintf(LOG,"%s:\tU in (%g,%g)\n",gen->genid,GEN->Umin,GEN->Umax);
+
+} /* end of _unur_dstd_debug_chg_truncated() */
+
 /*---------------------------------------------------------------------------*/
 #endif   /* end UNUR_ENABLE_LOGGING */
 /*---------------------------------------------------------------------------*/
@@ -651,6 +981,7 @@ _unur_dstd_info( struct unur_gen *gen, int help )
 {
   struct unur_string *info = gen->infostr;
   int samplesize = 10000;
+  int i;
 
   /* generator ID */
   _unur_string_append(info,"generator ID: %s\n\n", gen->genid);
@@ -685,6 +1016,31 @@ _unur_dstd_info( struct unur_gen *gen, int help )
     _unur_string_append(info,"\n");
   }
 
+  /* tables */
+  if (help) {
+    _unur_string_append(info,"table of precomputed double constants: ");
+    if (GEN->gen_param) {
+      _unur_string_append(info,"%d\n",GEN->n_gen_param);
+      for (i=0; i < GEN->n_gen_param; i++)
+  	_unur_string_append(info,"   [%d] = %g\n",i,GEN->gen_param[i]);
+    }
+    else  {
+      _unur_string_append(info,"none\n");
+    }
+
+    _unur_string_append(info,"table of precomputed integer constants: ");
+    if (GEN->gen_iparam) {
+      _unur_string_append(info,"%d\n",GEN->n_gen_iparam);
+      for (i=0; i < GEN->n_gen_iparam; i++)
+  	_unur_string_append(info,"   [%d] = %d\n",i,GEN->gen_iparam[i]);
+    }
+    else  {
+      _unur_string_append(info,"none\n");
+    }
+
+    _unur_string_append(info,"\n");
+  }
+
   /* Hints */
   /*   if (help) { */
   /*     _unur_string_append(info,"\n"); */
