@@ -1,6 +1,6 @@
 /*****************************************************************************
  *                                                                           *
- *          UNURAN -- Universal Non-Uniform Random number generator          *
+ *          UNU.RAN -- Universal Non-Uniform Random number generator         *
  *                                                                           *
  *****************************************************************************
  *                                                                           *
@@ -15,7 +15,7 @@
  *                                                                           *
  *****************************************************************************
  *                                                                           *
- *   Copyright (c) 2000-2008 Wolfgang Hoermann and Josef Leydold             *
+ *   Copyright (c) 2008-2010 Wolfgang Hoermann and Josef Leydold             *
  *   Department of Statistics and Mathematics, WU Wien, Austria              *
  *                                                                           *
  *   This program is free software; you can redistribute it and/or modify    *
@@ -40,9 +40,9 @@
 
    =UP  Methods_for_CONT
 
-   =REQUIRED PDF or CDF, center
+   =REQUIRED PDF
 
-   =OPTIONAL domain
+   =OPTIONAL domain, center, CDF, derivative of PDF
 
    =REF [DHLa08]
 
@@ -56,9 +56,9 @@
       The interval [0,1] is split into several subintervals. In each
       of these the inverse CDF is constructed at nodes 
       @unurmath{(CDF(x),x)} for some points @i{x} in this subinterval.
-      If the PDF is given then the CDF is computed numerically
+      If the PDF is given, then the CDF is computed numerically
       from the given PDF using adaptive Gauss-Lobatto
-      integration with 5 points. Subintervals are splitted until the
+      integration with 5 points. Subintervals are split until the
       requested accuracy goal is reached.
 
       The method is not exact, as it only produces random variates of
@@ -110,45 +110,69 @@
       an abortion of the set-up or (even worse) the approximation
       error might become larger than requested, since the (computation of the)
       interpolating polynomial becomes numerically unstable.
+      
+      @emph{Remark:}
+      We also have implemented experimental variants.
+      However, we observed that these variants are more sensitive to
+      round-off errors, especially in the right hand tail and we
+      @emph{do not recommend} their usage unless there are severe
+      reasons.
+
+      @itemize @minus
+      @item
+      Use a function that implements the CDF instead of numerical
+      integration of the PDF. 
+      
+      @item
+      Use Hermite interpolation instead of Newton interpolation.
+      Thus the first (and second) derivative of the interpolating
+      polynomial coincides with that of the inverse CDF.
+      Consequently the interpolant is also (twice) differentiable even
+      at the interval boundaries.
+      This variant can be seen as limiting case of Newton
+      interpolation with double (or triple) points as nodes.
+
+      We have used a @emph{smoothness} parameter to control this
+      feature. However, besides numerical problems we observed that
+      this variant requires more intervals and thus larger setup times
+      and higher memory consumptions.
+      @end itemize
 
 
    =HOWTOUSE
       PINV works for continuous univariate distribution objects with
-      given PDF. The corresponding distribution object must contain a
+      given PDF. The corresponding distribution object should contain a
       typical point of the distribution, i.e., a point where the PDF
       is not too small, e.g., (a point near) the mode.
       However, it is important that the center is @strong{not} the
       pole of the distribution (or a point too close to the pole).
       It can be set using a unur_distr_cont_set_center() or
-      a unur_distr_cont_set_mode() call. (If neither is set, @code{0}
-      is assumed!)
+      a unur_distr_cont_set_mode() call. If neither is set, or if the 
+      given center cannot be used, then a simple search routine tries
+      to find an appropriate point for the center. 
+
       It is recommended that the domain of the distribution with
       bounded domain is specified using a unur_distr_cont_set_domain()
       call. Otherwise, the boundary is searched numerically which
       might be rather expensive, especially when this boundary point
       is @code{0}.
       
-      When sampling from truncated distributions with extreme
-      truncation points, it is recommended to provide the log-density 
-      using unur_distr_cont_set_logpdf() and the mode.
-      Then the PDF is rescaled such that the PDF at the mode is 1.
-      Thus the algorithm is numerically more stable.
-
-      It is also possible to use the CDF of the distribution instead
-      of the PDF. Then the distribution object must contain a pointer
-      to the CDF. Moreover, this variant of the algorithmus has to be
-      switched on using an unur_pinv_set_usecdf() call.
-      Notice, however, that the setup for this variant is numerically
-      less stable than using integration of the PDF (the default
-      variant).
-
-      The inverse CDF is interpolated using Newton polymials.
+      The inverse CDF is interpolated using Newton polynomials.
       The order of this polynomial can be set by means of a
       unur_pinv_set_order() call.
+      
+      The smoothness of the interpolant at interval boundaries can be
+      controlled using a unur_pinv_set_smoothness() call. 
+      Then Hermite interpolation instead of Newton interpolation is
+      used. (The former can be seen as a limiting case of Newton
+      interpolation with double (or triple) points.)
+      However, using higher smoothness is @emph{not recommended}
+      unless differentiability at the interval boundaries is
+      important.
 
       For distributions with unbounded domains the tails are cut
       off such that the probability for the tail regions is small
-      compared to the given u-resulution. For finding these cut points
+      compared to the given u-resolution. For finding these cut points
       the algorithm starts with the region @code{[-1.e100,1.e100]}. For
       the exceptional case where this does not work these starting
       points can be changed via a unur_pinv_set_boundary() call.
@@ -177,6 +201,14 @@
       If this maximum number is too small then the set-up aborts with
       a corresponding error message.
 
+      It is also possible to use the CDF of the distribution instead
+      of the PDF. Then the distribution object must contain a pointer
+      to the CDF. Moreover, this variant of the algorithm has to be
+      switched on using an unur_pinv_set_usecdf() call.
+      Notice, however, that the setup for this variant is numerically
+      less stable than using integration of the PDF (the default
+      variant). Thus using the CDF is @emph{not recommended}.
+
    =END
 */
 
@@ -198,12 +230,73 @@ UNUR_PAR *unur_pinv_new( const UNUR_DISTR *distribution );
 int unur_pinv_set_order( UNUR_PAR *parameters, int order);
 /* 
    Set order of interpolation. Valid orders are between @code{3} and
-   @code{12}. Higher orders result in fewer intervals for the
+   @code{17}. Higher orders result in fewer intervals for the
    approximations. 
 
    Default: @code{5}.
 */
 
+int unur_pinv_set_smoothness( UNUR_PAR *parameters, int smoothness);
+/* 
+   Set smoothness of interpolant. By construction the interpolant is
+   piecewise polynomial and thus smooth on each of the intervals
+   where these polynomials are constructed. At the interval
+   boundaries, however, it usually not be differentiable.
+   Method PINV also implements variants of Newton interpolation where
+   the first (or second) derivative of the interpolating
+   polynomial coincides with the respective derivative of the inverse
+   CDF at the nodes. The the interpolant is (twice) differentiable
+   even at the interval boundaries.
+   These variants can be seen as limiting case of Newton interpolation
+   with double (or triple) points as nodes and are known as Hermite
+   interpolation.
+
+   Possible values for @var{smoothness}:
+
+   @multitable @columnfractions .1 .25 .60
+   @headitem Value @tab Effect @tab Requirements
+   @item @code{0} 
+   @tab continuous
+   @tab requires PDF (or CDF)
+
+   @item @code{1} 
+   @tab differentiable
+   @tab requires PDF (optional: CDF), @*
+   order of polynomial must be odd
+
+   @item @code{2} 
+   @tab twice differentiable
+   @tab requires PDF and its derivative (optional: CDF), @*
+   order must be 5, 8, 11, 14 or 17
+   @end multitable
+
+   If the order of the polynomial does not satisfy the given
+   condition, then it is increased to the next larger possible value.
+
+   @emph{Remark:}
+   A higher smoothness parameter usually results in a higher number of
+   intervals and thus a higher setup time and memory consumption.
+   We also observed that higher smoothness parameters make the
+   algorithm more sensible for round-off error. Then the setup fails.
+
+   @emph{Remark:}
+   If the interpolating polynomial cannot be constructed for the
+   requested smoothness on a particular interval, 
+   then the smoothness parameter is reduced for that interval.
+
+   @emph{Remark:}
+   For order @code{3} and smoothness @code{1} (cubic Hermite
+   interpolation) monotonicity is guaranteed by checking a simple
+   monotonicity condition for the coefficients of the polynomials.
+
+   @emph{Remark:}
+   Using @var{smoothness} larger than @code{0} is 
+   @emph{not recommended} unless differentiability at the interval
+   boundaries is important for ones application.
+
+   Default: @code{0}.
+*/
+
 int unur_pinv_set_u_resolution( UNUR_PAR *parameters, double u_resolution);
 /* 
    Set maximal tolerated u-error. Values of @var{u_resolution} must
@@ -229,13 +322,24 @@ int unur_pinv_set_u_resolution( UNUR_PAR *parameters, double u_resolution);
    can be as large as @code{1.e-14}.
 
    @strong{Warning!}
-   These figures are based on our experiments (with some tolarence
-   added to be on the safe side). There is no guarentee for these error
+   These figures are based on our experiments (with some tolerance
+   added to be on the safe side). There is no guarantee for these error
    estimates for a particular distribution.
 
    Default is @code{1.e-10}.
 */
 
+int unur_pinv_set_use_upoints( UNUR_PAR *parameters, int use_upoints );
+/* 
+   If @var{use_upoints} is TRUE, then the nodes of the interpolating
+   polynomial are constructed by means of Chebyshev points in u-scale
+   not in x-scale. This results is a better approximation but almost
+   doubles the number of PDF or CDF evaluations during the setup.
+   (This is an experimental feature.)
+   
+   Default: FALSE
+*/
+
 int unur_pinv_set_usepdf( UNUR_PAR *parameters );
 /* 
    Use PDF (if available) to compute approximate inverse CDF.
@@ -246,12 +350,17 @@ int unur_pinv_set_usepdf( UNUR_PAR *parameters );
 int unur_pinv_set_usecdf( UNUR_PAR *parameters );
 /* 
    Use CDF (if available) to compute approximate inverse CDF.
+   This variant is intend for running experiments with method PINV.
 
    @emph{Remark:}
    We ran many experiments and found that for small values of the
    given @var{u_resolution} (less than @code{1.e-12}) the setup fails
    for distributions with heavy tails. We found that using the PDF
    (instead of the CDF) is numerically more stable.
+   This is especially the case when the smoothness parameter is set
+   to @code{1} or @code{2}.
+
+   Using the CDF is @strong{not recommended}.
 */
 
 int unur_pinv_set_boundary( UNUR_PAR *parameters, double left, double right );
@@ -310,6 +419,21 @@ int unur_pinv_get_n_intervals( const UNUR_GEN *generator );
    It returns @code{0} in case of an error.
 */
 
+int unur_pinv_set_keepcdf( UNUR_PAR *parameters, int keepcdf);
+/* 
+   If the PDF is given, then the CDF is computed numerically
+   from the given PDF using adaptive Gauss-Lobatto integration.
+   Thus a table of CDF points is stored to keep the number of 
+   evaluations of the PDF minimal. Usually this table is discarded
+   when the setup is completed.
+   If @var{keepcdf} is TRUE, then this table is kept and can be used
+   to compute the CDF of the underlying distribution by means of 
+   function unur_pinv_eval_approxcdf().
+   This option is ignored when unur_pinv_set_usecdf() is called.
+   
+   Default: FALSE
+*/
+
 double unur_pinv_eval_approxinvcdf( const UNUR_GEN *generator, double u );
 /*
    Evaluate interpolation of inverse CDF at @var{u}.
@@ -320,6 +444,16 @@ double unur_pinv_eval_approxinvcdf( const UNUR_GEN *generator, double u );
    unbounded domains).
 */
 
+double unur_pinv_eval_approxcdf( const UNUR_GEN *generator, double x );
+/*
+   Evaluate (approximate) CDF at @var{x}. If the PDF of the
+   distribution is given, then adaptive Gauss-Lobatto integration is
+   used to compute the CDF.
+   If the PDF is used to create the generator object, then the
+   table of integral values must not removed at the end of setup and thus 
+   unur_pinv_set_keepcdf() must be called.
+*/
+
 int unur_pinv_estimate_error( const UNUR_GEN *generator, int samplesize, double *max_error, double *MAE );
 /*
    Estimate maximal u-error and mean absolute error (MAE) for @var{generator}
