@@ -37,7 +37,7 @@ use strict;
 my $VERBOSE = 1;
 
 # ----------------------------------------------------------------------------
-# $Id: make_stringparser.pl 4867 2010-01-29 07:45:59Z leydold $
+# $Id: make_stringparser.pl 5535 2011-12-22 01:49:38Z leydold $
 # ----------------------------------------------------------------------------
 #
 # File: make_stringparser.pl
@@ -323,7 +323,7 @@ sub make_list_of_distributions {
     $code .= "\t }\n\n";
 
     # print docu
-    $distr_doc_string .= "\@end itemize\n\n\@sp 1\n";
+    $distr_doc_string .= "\@end itemize\n\n";
 
     print STDERR "\n\n";
 
@@ -368,7 +368,7 @@ sub make_list_of_distributions {
     $code .= "\t }\n\n";
 
     # print docu
-    $distr_doc_string .= "\@end itemize\n\n\@sp 1\n";
+    $distr_doc_string .= "\@end itemize\n\n";
 
     # add comment for order statistics ...
     $distr_doc_string .= comment_for_corder();
@@ -686,7 +686,7 @@ sub make_distr_set_calls {
     }
 
     # end of table
-    $distr_doc_string .= "\@end table\n\n\@sp 1\n";
+    $distr_doc_string .= "\@end table\n\n";
 
     # Return result
     return $code;
@@ -1017,7 +1017,7 @@ sub make_list_of_par_sets {
 	$code .= "\t\t break;\n";
 
 	# print docu
-	$method_doc_string .= "\@end table\n\n\@sp 1\n";
+	$method_doc_string .= "\@end table\n\n";
     }
 
     # end of switch for methods
@@ -1067,8 +1067,6 @@ Order statistics for continuous distributions (\@pxref{CORDER}) are
 supported by using the key \@code{orderstatistics} for distributions
 of type \@code{CONT}.
 
-\@sp 1
-
 EOS
 
 } # end if comment_for_corder()
