@@ -45,7 +45,6 @@
 #include <distr/discr.h>
 #include <distr/distr_source.h>
 #include <distributions/unur_distributions.h>
-#include <specfunct/unur_specfunct_source.h>
 #include <utils/matrix_source.h>
 #include "unuran_tests.h"
 
@@ -239,7 +238,7 @@ _unur_test_chi2_discr( struct unur_gen *gen,
     /* shift vector */
     j -= DISTR.domain[0];
     /* check range of random variates !! */
-    if (j < n_pv)
+    if (j >= 0 && j < n_pv) 
       ++observed[j];
     /* else: ignore number --> chop off tail */
   }
@@ -337,12 +336,12 @@ _unur_test_chi2_cont( struct unur_gen *gen,
 
   /* compute Fl and Fr */
   if (gen->distr->set & UNUR_DISTR_SET_TRUNCATED) {
-    Fl = (DISTR.trunc[0] <= -INFINITY) ? 0. : cdf(DISTR.trunc[0], gen->distr);
-    Fr = (DISTR.trunc[1] >=  INFINITY) ? 1. : cdf(DISTR.trunc[1], gen->distr);
+    Fl = (DISTR.trunc[0] <= -UNUR_INFINITY) ? 0. : cdf(DISTR.trunc[0], gen->distr);
+    Fr = (DISTR.trunc[1] >=  UNUR_INFINITY) ? 1. : cdf(DISTR.trunc[1], gen->distr);
   }
   else {
-    Fl = (DISTR.domain[0] <= -INFINITY) ? 0. : cdf(DISTR.domain[0], gen->distr);
-    Fr = (DISTR.domain[1] >=  INFINITY) ? 1. : cdf(DISTR.domain[1], gen->distr);
+    Fl = (DISTR.domain[0] <= -UNUR_INFINITY) ? 0. : cdf(DISTR.domain[0], gen->distr);
+    Fr = (DISTR.domain[1] >=  UNUR_INFINITY) ? 1. : cdf(DISTR.domain[1], gen->distr);
   }
   Fdelta = Fr - Fl;
 
@@ -769,10 +768,6 @@ _unur_test_chi2_cvemp ( struct unur_gen *gen,
   CHECK_NULL(gen,-1.);
   /* we do not check magic cookies here */
 
-  /* CDF for standard normal distribution */
-  distr_normal = unur_distr_normal( NULL, 0 );
-  cdf = distr_normal->data.cont.cdf;
-
   /* check given number of intervals */
   if (n_intervals <= 2)
     n_intervals = CHI2_INTERVALS_DEFAULT;
@@ -788,6 +783,10 @@ _unur_test_chi2_cvemp ( struct unur_gen *gen,
     return -1.;
   }
 
+  /* we need the CDF for the standard normal distribution */
+  distr_normal = unur_distr_normal( NULL, 0 );
+  cdf = distr_normal->data.cont.cdf;
+
   /* allocate working space memory */
   X   = _unur_xmalloc( dim * sizeof(double));
   U   = _unur_xmalloc( dim * sizeof(double));
@@ -832,9 +831,9 @@ _unur_test_chi2_cvemp ( struct unur_gen *gen,
 
   /* free memory */
   _unur_distr_free(distr_normal);
-  if (X)  free(X);
-  if (U)  free(U);
-  if (bm) free(bm);
+  free(X);
+  free(U);
+  free(bm);
 
   /* return result of test */
   return pval_min * dim;
